#ifndef _DataBlock_h
#define _DataBlock_h 1
/**************************************************************************
Author:  Eric K. Pauer

$Id: DataBlock.h,v 1.3 1995/07/05 19:06:31 epauer Exp $

This class is contains a packet for the Architecture trade effort.  It
stores a representation of the data (type of data and quantity), the 
device to which the packet is to be routed, and a list of ports to be
used to send the data.  There are two constructors, one for a single port
and one to handle the case of more than one port in the route list.

This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
**************************************************************************/
#ifdef __GNUG__
#pragma interface
#endif

#include "Message.h"

class DataBlock : public Message
  {
  public:
    // constructor
    DataBlock(int size, const char *dataRepresention, int processor, int functionBlock, int portCount, int *portList);

    // constructor
    DataBlock(int size, const char *dataRepresentation, int processor, int functionBlock, int port);

    // copy constructor
    DataBlock(const DataBlock& src);

    // destructor
    ~DataBlock() {};

    // clone
    Message* clone() const;

    StringList print() const;

    int dataSize() const;
    const char * dataRepresentation() const;
    int processor() const;
    int functionBlock() const;
    int portCount() const;
    int portList(int i) const;

    void setDataSize(int dataSize);
    void setDataRepresentation(const char *dataRepresentation);
    void setProcessor(int processor);
    void setFunctionBlock(int functionBlock);
    void setPortCount(int portCount);
    void setPortList(int index, int port);

    // virtual functions
    const char* dataType() const;
    int isA(const char*) const;

  private:
    int dataSize_;
    char dataRepresentation_[7];
    int processor_;
    int functionBlock_;
    int portCount_;
    int portList_[5];
};
#endif
