#ifndef _ResourceBlock_h
#define _ResourceBlock_h 1
/**************************************************************************
Author:  Eric K. Pauer

$Id: ResourceBlock.h,v 1.2 1995/07/18 10:54:06 epauer Exp $

This class is contains a packet for the Architecture trade effort.  It
stores a resource request that is passed between the function block and
the processor to which it has been mapped.

This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
**************************************************************************/
#ifdef __GNUG__
#pragma interface
#endif

#include "Message.h"

class ResourceBlock : public Message
  {
  public:
    // constructor
    ResourceBlock(const char *functionName, int functionNumber, int cycles, int memory, int vectorSize);

    // copy constructor
    ResourceBlock(const ResourceBlock& src);

    // destructor
    ~ResourceBlock() {};

    // clone
    Message* clone() const;

    StringList print() const;

    int cycles() const;
    int memory() const;
    int vectorSize() const;
    const char* functionName() const;
    int functionNumber() const;

    void setCycles(int cycles);
    void setMemory(int memory);
    void setVectorSize(int vectorSize);
    void setFunctionName(const char* functionName);
    void setFunctionNumber(int functionNumber);

    // virtual functions
    const char* dataType() const;
    int isA(const char*) const;

  private:
    char functionName_[32];
    int functionNumber_;
    int cycles_;
    int memory_;
    int vectorSize_;
};
#endif
