defstar {
	name {Bus}
	domain {DE}
	version { $Revision: 1.8 $ $Date: 1995/07/19 18:46:22 $ }
	author { E. K. Pauer }
	copyright {
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
	}
	acknowledge {
This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
}
	location { $PTOLEMY/src/contrib/atrade/de/stars }
	desc { Models a bus by coordinating resources (bandwidth and users) }
	inmulti {
		name {inputPort}
		type {message}
	}
	outmulti {
		name {outputPort}
		type {message}
	}
	defstate {
		name {partNumber}
		type {string}
		default {""}
		desc { Type of bus }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {busNumber}
		type {int}
		default {0}
		desc { Unique id of the bus }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {bandwidth}
		type {int}
		default {1000000}
		desc { Clock rate of the processor in words per second}
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {busWidth}
		type {int}
		default {32}
		desc { Width of bus in bits }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {simultaneousUsers}
		type {int}
		default {1}
		desc { Number of users who can access the bus simultaneously }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	hinclude { <iostream.h>, <fstream.h>, "Message.h", "DataBlock.h" }
	ccinclude { "DEProcessor.h" }
	constructor {
	  delayType = TRUE;
	}
	protected {
	  InDEPort* source;
	  OutDEPort* dest;
	  double delay;
	  int i, j;
	  double* busy;
	  int wait;
	  double earliestAvailable;
	  int earliestAvailableIndex;
	  ofstream clog;
	}
	begin {
	  busy = new double[simultaneousUsers];
	  for (i = 0; i < simultaneousUsers; i++)
            busy[i] = double(0);

     	  clog.open(DEProcessor::logname, ios::app);
          clog << this->fullName() << ": ";
	  clog << "inputPorts = " << inputPort.numberPorts() << ", outputPorts = " << outputPort.numberPorts() << endl;
	  clog.close();
	}
	go
	  {
	  // check if data has arrived on the internalInput
          InDEMPHIter sourceIter(inputPort);
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    {
	    source = sourceIter++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
	        return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      delay = double(int(dataBlockPtr->dataSize()) * 4) / (double(busWidth / 8) * double(bandwidth));
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " received on inputPort " << i << " at time ";
	      clog << arrivalTime << ", transfer time of " << delay << " calculated" << endl;
	      clog.close();
	      wait = 1;
	      earliestAvailable = busy[0];
	      earliestAvailableIndex = 0;
	      // check for available time slot
	      for (j = 0; j < simultaneousUsers; j++)
		{
		if (arrivalTime > busy[j])
		  {
		  // resource available, reserve for appropriate time slot
		  busy[j] = arrivalTime + delay;
		  completionTime = busy[j];
		  wait = 0;
		  break;
		  }
		else
		  // resource not available, keep track of earilest available time slot
		  if (busy[j] < earliestAvailable)
		    {
		    earliestAvailable = busy[j];
	            earliestAvailableIndex = j;
		    }
		}
	      // if limit has been reached, utilize first available time slot
              if (wait)
		{
		busy[earliestAvailableIndex] += delay;
		completionTime = busy[earliestAvailableIndex];
		}
	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", transmission of a data block of " << dataBlockPtr->dataSize();
	      clog << " samples of " << dataBlockPtr->dataRepresentation() << " to " << outputPort.numberPorts();
	      clog << " processors on this bus completed at time " << completionTime << endl;
	      clog.close();
	      ofstream outFile(DEProcessor::filename, ios::app);
	      if (!outFile)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << DEProcessor::filename << endl;
	        Error::abortRun(*this,"Unable to open output file.");
	        }
	      outFile << "B" << busNumber << " " << dataBlockPtr->processor() << " " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      // send data to all outputs
              OutDEMPHIter destIter(outputPort);
	      for (j = 0; j < outputPort.numberPorts(); j++)
		{
		dest = destIter++;
		dest->put(completionTime) << pkt;
		}
	      }
	    }
	  }
}
