static const char file_id[] = "../../../../../src/contrib/atrade/de/stars/DEFunction.pl";
// .cc file generated from ../../../../../src/contrib/atrade/de/stars/DEFunction.pl by ptlang
/*
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEFunction.h"
#include "DEProcessor.h"

const char *star_nm_DEFunction = "DEFunction";

const char* DEFunction :: className() const {return star_nm_DEFunction;}

ISA_FUNC(DEFunction,DEStar);

Block* DEFunction :: makeNew() const { LOG_NEW; return new DEFunction;}

DEFunction::DEFunction ()
{
	setDescriptor("Models the performance of an SDF star based on cycles and memory needed.");
	addPort(inputPort.setPort("inputPort",this,MESSAGE));
	addPort(outputPort.setPort("outputPort",this,MESSAGE));
	addPort(acknowledge.setPort("acknowledge",this,MESSAGE));
	addPort(request.setPort("request",this,MESSAGE));
	addState(functionName.setState("functionName",this,"","Name of function implemented",
# line 42 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(functionNumber.setState("functionNumber",this,"0","Unique id of the functional block"));
	addState(inputSamples.setState("inputSamples",this,"1","DataBlocks needed on each input for star to fire",
# line 55 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(outputSamples.setState("outputSamples",this,"1","DataBlocks produced on each output per each firing",
# line 62 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(inputDataType.setState("inputDataType",this,"FLOAT","Type of data accepted by each inputPort",
# line 69 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(outputDataType.setState("outputDataType",this,"FLOAT","Type of data produced by each outputPort",
# line 76 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(processingCostOverhead.setState("processingCostOverhead",this,"15","number of overhead CPU cycles needed to fire",
# line 83 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(processingUnitCost.setState("processingUnitCost",this,"10","number of CPU cycles needed to fire which is scalable by the amount of data",
# line 90 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(memoryCostOverhead.setState("memoryCostOverhead",this,"20","bytes of overhead memory needed to fire",
# line 97 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(memoryUnitCost.setState("memoryUnitCost",this,"8","bytes of overhead memory needed to fire which is dependent of the amount of data",
# line 104 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(vectorizable.setState("vectorizable",this,"1","specifies whether or not function can process more than one data particle per firing",
# line 111 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(processorMapping.setState("processorMapping",this,"0","List of processors (by number) of the function blocks mapped to outputPort",
# line 118 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(functionBlockMapping.setState("functionBlockMapping",this,"0","List of function blocks (by number) mapped to outputPort",
# line 125 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));
	addState(inputPortMapping.setState("inputPortMapping",this,"0","List of inputPorts (by number) of the functional blocks mapped to outputPort",
# line 132 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
A_CONSTANT | A_SETTABLE));

# line 155 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
delayType = TRUE;
	  awaitingAcknowledge = 0;
}

void DEFunction::begin() {
# line 159 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
queuedCount = new int[inputPort.numberPorts()];
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    queuedCount[i] = 0;
     	  clog.open(DEProcessor::logname, ios::app);
          clog << this->fullName() << ": inputPorts = " <<  inputPort.numberPorts() << ", outputPorts = " << outputPort.numberPorts() << endl;
	  clog.close();

	  inSamples = new int[inputPort.numberPorts()];
	  if (inputSamples.size() == 1)
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      inSamples[i] = inputSamples[0];
	  else
	    {
	    if (inputSamples.size() != inputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of inputSamples specified does not match number of inputPorts" << endl;
	      Error::abortRun(*this, "Number of inputSamples specified does not match number of inputPorts");
	      }	      
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      inSamples[i] = inputSamples[i];
	    }

	  outSamples = new int[outputPort.numberPorts()];
	  if (outputSamples.size() == 1)
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      outSamples[i] = outputSamples[0];
	  else
	    {
	    if (outputSamples.size() != outputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of outputSamples specified does not match number of outputPorts" << endl;
	      Error::abortRun(*this, "Number of outputSamples specified does not match number of outputPorts");
	      }
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      outSamples[i] = outputSamples[i];
	    }

	  inDataType = new CharPtr[inputPort.numberPorts()];
	  if (inputDataType.size() == 1)
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      {
	      inDataType[i] = new char[strlen(inputDataType[0])];
	      strcpy(inDataType[i], inputDataType[0]);
	      }
	  else
	    {
	    if (inputDataType.size() != inputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of inputDataTypes specified does not match number of inputPorts" << endl;
	      Error::abortRun(*this, "Number of inputDataTypes specified does not match number of inputPorts");
	      }	      
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      {
	      inDataType[i] = new char[strlen(inputDataType[i])];
	      strcpy(inDataType[i], inputDataType[i]);
	      }
	    }


	  outDataType = new CharPtr[outputPort.numberPorts()];
	  if (outputDataType.size() == 1)
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      outDataType[i] = new char[strlen(outputDataType[0])];
	      strcpy(outDataType[i], outputDataType[0]);
	      }
	  else
	    {
	    if (outputDataType.size() != outputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of outputDataTypes specified does not match number of outputPorts" << endl;
	      Error::abortRun(*this, "Number of outputDataTypes specified does not match number of outputPorts");
	      }	      
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      outDataType[i] = new char[strlen(outputDataType[i])];
	      strcpy(outDataType[i], outputDataType[i]);
	      }
	    }
}

void DEFunction::go() {
# line 240 "../../../../../src/contrib/atrade/de/stars/DEFunction.pl"
// check to see if acknowledge arrived
	  if (acknowledge.dataNew)
	    {
	    Envelope pkt;
	    (acknowledge.get()).getMessage(pkt);
	    if (!pkt.typeCheck("ResourceBlock"))
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", typeError(ResourceBlock)" << endl;
	      Error::abortRun(*this, pkt.typeError("ResourceBlock"));
	      return;
	      }
            const ResourceBlock* resourceBlockPtr = (const ResourceBlock *)pkt.myData();
	    completionTime = arrivalTime;
     	    clog.open(DEProcessor::logname, ios::app);
	    clog << this->fullName() << ", acknowledge to process " << resourceBlockPtr->vectorSize();
	    clog << " data blocks (particles) received at time " << arrivalTime << endl;
	    clog.close();

	    // "consume" DataBlocks on input queues
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      queuedCount[i] -= inSamples[i];

	    // send data out each outputPort, scaled by the vectorSize
	    OutDEMPHIter destIter(outputPort);
	    DataBlock* dataBlockPtr;
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      dest = destIter++;
 	      dataBlockPtr = new DataBlock((resourceBlockPtr->vectorSize() * outSamples[i]), outDataType[i],
                                              processorMapping[i], functionBlockMapping[i], inputPortMapping[i]);
	      Envelope pkt(*dataBlockPtr);
	      dest->put(completionTime) << pkt;
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
              clog << dataBlockPtr->dataRepresentation() << " sent out outputPort " << i << " to functionBlock ";
	      clog << functionBlockMapping[i] << " (inputPort " << inputPortMapping[i] << ") on processor ";
	      clog << processorMapping[i] << " at time " << completionTime << endl;
	      clog.close();
	      }

	    // no longer waiting for an acknowledge
	    awaitingAcknowledge = 0;
	    }

	  // else data has arrived on one or more inputPort
          InDEMPHIter sourceIter(inputPort);
	  for (i = 0; i < inputPort.numberPorts(); i++) {
	    source = sourceIter++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
		return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      if (dataBlockPtr->functionBlock() != functionNumber)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received (wrong function)" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received (wrong function)");
	        return;
	        }
	      if ((dataBlockPtr->portList(0) < 0) || (dataBlockPtr->portList(0) >= inputPort.numberPorts()))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received (right function, bad port)" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received (right function, bad port)");
	        return;
	        }
	      if (strcmp(dataBlockPtr->dataRepresentation(), inDataType[i]) != 0)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", DataBlock with wrong data type received" << endl;
	        Error::abortRun(*this,"DataBlock with wrong data type received");
	        return;
	        }
	      queuedCount[dataBlockPtr->portList(0)] += inSamples[i];
	      // temporary assumption, assumes all blocks in and out are same size
	      outSize = dataBlockPtr->dataSize();
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " arrived on inputPort " << dataBlockPtr->portList(0) << " at time " << arrivalTime << endl;
	      clog.close();
	      }
	  }	    

	  // exit if there is not enough DataBlocks on all ports
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    if (queuedCount[i] < inSamples[i])
	      return;

	  // exit if waiting for an acknowledge
	  if (awaitingAcknowledge)
	      return;

	  // enough data is now available on all input ports to fire

/*
	  if (vectorizable)
	    {
	    // check for vectorization
	    vector = queuedCount[0] / inSamples[0];
	    for (i = 1; i < inputPort.numberPorts(); i++)
	      {
	      // calculate number of firings that can be combined (integer division)
	      temp = queuedCount[i] / inSamples[i];
	      if (temp < vector)
	        vector = temp;
	      }
	    }
	  else
	    // else process one data block (particle)
*/
	  // currently not used
	  vector = 1;

	  // send request for processor for processing and memory resources
	  processingCost = int(processingCostOverhead) + int(processingUnitCost) * vector;
	  memoryCost = int(memoryCostOverhead) + int(memoryUnitCost) * vector;
	  ResourceBlock* resourceBlockPtr = new ResourceBlock(functionName, functionNumber, processingCost, memoryCost, outSize);
	  Envelope pkt(*resourceBlockPtr);
	  completionTime = arrivalTime;
	  request.put(completionTime) << pkt;
	  awaitingAcknowledge = 1;
     	  clog.open(DEProcessor::logname, ios::app);
	  clog << this->fullName() << ", request to process " << vector << " data blocks (particles) sent at time " << completionTime << endl;
	  clog.close();
}

// prototype instance for known block list
static DEFunction proto;
static RegisterBlock registerBlock(proto,"Function");
