static const char file_id[] = "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl";
// .cc file generated from ../../../../../src/contrib/atrade/de/stars/DEProcessor.pl by ptlang
/*
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEProcessor.h"

const char *star_nm_DEProcessor = "DEProcessor";

const char* DEProcessor :: className() const {return star_nm_DEProcessor;}

ISA_FUNC(DEProcessor,DEStar);

Block* DEProcessor :: makeNew() const { LOG_NEW; return new DEProcessor;}

# line 112 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
const char* DEProcessor::filename = "/usr/tmp/gantt.dat";
	  const char* DEProcessor::logname = "/usr/tmp/de_log.dat";

DEProcessor::DEProcessor ()
{
	setDescriptor("Models the a processor by coordinating resources (CPU and memory).");
	addPort(externalInput.setPort("externalInput",this,MESSAGE));
	addPort(externalOutput.setPort("externalOutput",this,MESSAGE));
	addPort(internalInput.setPort("internalInput",this,MESSAGE));
	addPort(internalOutput.setPort("internalOutput",this,MESSAGE));
	addPort(request.setPort("request",this,MESSAGE));
	addPort(acknowledge.setPort("acknowledge",this,MESSAGE));
	addState(partNumber.setState("partNumber",this,"","Type of processor by part number",
# line 50 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(processorNumber.setState("processorNumber",this,"0","Unique id of the processor",
# line 57 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(clockRate.setState("clockRate",this,"1000000","Clock rate of the processor",
# line 64 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(inputSampleTransferRate.setState("inputSampleTransferRate",this,"10000000","Transfer rate of data at input",
# line 71 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(outputSampleTransferRate.setState("outputSampleTransferRate",this,"10000000","Transfer rate of data at output",
# line 78 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(functionBlockMapping.setState("functionBlockMapping",this,"0","List of function blocks (by number) mapped to internalOutput",
# line 85 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));
	addState(inputPortMapping.setState("inputPortMapping",this,"0","List of input ports (by number) of the functional blocks mapped to internalOutput",
# line 92 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
A_CONSTANT | A_SETTABLE));

# line 96 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
delayType = TRUE;
}

void DEProcessor::begin() {
# line 116 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
clog.open(logname, ios::app);
          clog << this->fullName() << ": internalInputs = " << internalInput.numberPorts();
	  clog << ", internalOutputs = " << internalOutput.numberPorts();
	  clog << ", requests = " << request.numberPorts() << ", acknowledges = " << acknowledge.numberPorts() << endl;
	  clog.close();

	  // check to insure there are consistent number of ports
	  if (request.numberPorts() != acknowledge.numberPorts())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Inconsistent number of request and acknowledge ports" << endl;
	    Error::abortRun(*this,"Inconsistent number of request and acknowledge ports");
	    }
	  // check to insure the correct number of mappings have been specified
	  if (internalOutput.numberPorts() != functionBlockMapping.size())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Incorrect number of function block mappings specified for internalInput" << endl;
	    Error::abortRun(*this,"Incorrect number of function block mappings specified for internalInput");
	    }
	  if (internalOutput.numberPorts() != inputPortMapping.size())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Incorrect number of inputPort mappings specified for internalInput" << endl;
	    Error::abortRun(*this,"Incorrect number of inputPort mappings specified for internalInput");
	    }
}

void DEProcessor::go() {
# line 142 "../../../../../src/contrib/atrade/de/stars/DEProcessor.pl"
// check if data has arrived on the externalInput
	  if (externalInput.dataNew)
	    {
	    Envelope pkt;
	    (externalInput.get()).getMessage(pkt);
	    if (!pkt.typeCheck("DataBlock"))
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	      Error::abortRun(*this, pkt.typeError("DataBlock"));
	      return;
	      }
            const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	    if (dataBlockPtr->processor() != processorNumber)
	      {
     	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " did not match this processor and has been ignored" << endl;
	      clog.close();
	      }
	    else
	      {
	      // calculate delay
	      delay = double(dataBlockPtr->dataSize()) / double(inputSampleTransferRate);
     	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " arrived on externalInput at time " << arrivalTime;
	      clog << ", transfer time of " << delay << " calculated" << endl;
	      clog.close();
	      if (arrivalTime > completionTime)
	        completionTime = arrivalTime + delay;
	      else
	        completionTime += delay;
	      ofstream outFile(filename, ios::app);
	      if (!outFile)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	        Error::abortRun(*this,"Unable to open output file.");
	        }
	      outFile << "P" << processorNumber << " t " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      found = 0;
	      // attempt to find index to portNumber in logical list for internalOutput
              for (portNumberIndex = 0; portNumberIndex < internalOutput.numberPorts(); portNumberIndex++)
	        {
	        if ((dataBlockPtr->functionBlock() == functionBlockMapping[portNumberIndex]) &&
		    (dataBlockPtr->portList(0) == inputPortMapping[portNumberIndex]))
		  {
		  found = 1;
		  break;
		  }
	        }
	      if (found)
	        {
	        // find correct port to send data using index
                OutDEMPHIter destIter1(internalOutput);
	        for (j = 0; j <= portNumberIndex; j++)
		  dest = destIter1++;
	        // send data to appropriate feedOutput
	        dest->put(completionTime) << pkt;
     	        clog.open(logname, ios::app);
	        clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
                clog << dataBlockPtr->dataRepresentation() << " sent out internalOutput ";
	        clog << portNumberIndex << " to functionBlock " << functionBlockMapping[portNumberIndex];
	        clog << " (inputPort " << inputPortMapping[portNumberIndex] << ") at time " << completionTime << endl;
		clog.close();
	        }
	      else
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received on externalInput" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received on externalInput");
		return;
	        }
	      }
	    }

	  // check if data has arrived on the internalInput
          InDEMPHIter sourceIter2(internalInput);
	  for (i = 0; i < internalInput.numberPorts(); i++)
	    {
	    source = sourceIter2++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
	        return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      // if data is not for this processor, send it out the externalOutput
	      if (dataBlockPtr->processor() != processorNumber)
	        {
	        delay = double(dataBlockPtr->dataSize()) / double(outputSampleTransferRate);
		clog.open(logname, ios::app);
	        clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
		clog << dataBlockPtr->dataRepresentation() << " arrived on internalInput " << i << " at time " << arrivalTime;
	        clog << ", transfer time of " << delay << " calculated" << endl;
		clog.close();
	        if (arrivalTime > completionTime)
	          completionTime = arrivalTime + delay;
	        else
	          completionTime += delay;
	        ofstream outFile(filename, ios::app);
	        if (!outFile)
	          {
	          cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	          Error::abortRun(*this,"Unable to open output file.");
	          }
	        outFile << "P" << processorNumber << " t " << (completionTime - delay) << " " << completionTime << endl;
	        outFile.close();
		externalOutput.put(completionTime) << pkt;
     	        clog.open(DEProcessor::logname, ios::app);
		clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
		clog << dataBlockPtr->dataRepresentation() << " sent out externalOutput at time " << completionTime << endl;
		clog.close();
	        }
	      // else data is for this processor, send out appropriate internalOutput port
	      else
		{
		found = 0;
	        // attempt to find index to portNumber in logical list for internalOutput
                for (portNumberIndex = 0; portNumberIndex < internalOutput.numberPorts(); portNumberIndex++)
	          if ((dataBlockPtr->portList(0) == inputPortMapping[portNumberIndex]) &&
		      (dataBlockPtr->functionBlock() == functionBlockMapping[portNumberIndex]))
		    {
		    found = 1;
		    break;
		    }
	        if (found)
	          {
	      	  // find correct port to send data using index
              	  OutDEMPHIter destIter2(internalOutput);
	      	  for (j = 0; j <= portNumberIndex; j++)
		    dest = destIter2++;
	      	  // send data to appropriate internalOutput, no delay incurred when passing data between functions on same processor
	      	  dest->put(arrivalTime) << pkt;
		  clog.open(logname, ios::app);
	          clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
                  clog << dataBlockPtr->dataRepresentation() << " sent out internalOutput " << portNumberIndex;
		  clog << " to functionBlock " << functionBlockMapping[portNumberIndex];
	          clog << " (inputPort " << inputPortMapping[portNumberIndex] << ") on processor ";
		  clog << dataBlockPtr->processor() << " at time " << arrivalTime << endl;	
		  clog.close();
      	  	  }
	    	else
		  {
	          cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received on internalInput" << endl;
		  Error::abortRun(*this, "Incorrectly addressed DataBlock received on internalInput");
		  return;
		  }
	        }
	      }
	    }

	  // Handle any requests for resources that have arrived
          InDEMPHIter sourceIter3(request);
          OutDEMPHIter destIter3(acknowledge);
	  for (i = 0; i < request.numberPorts(); i++)
	    {
	    source = sourceIter3++;
	    dest = destIter3++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
      	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("ResourceBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(ResourceBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("ResourceBlock"));
	        return;
	        }
              const ResourceBlock* resourceBlockPtr = (const ResourceBlock *)pkt.myData();
	      delay = double(resourceBlockPtr->cycles()) / double(clockRate);
	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", request to process " << resourceBlockPtr->vectorSize();
	      clog << " data blocks (particles) (" << resourceBlockPtr->cycles() << " cycles, " << resourceBlockPtr->memory();
	      clog << " bytes) arrived on request " << i << " at time " << arrivalTime;
	      clog << ", processing time of " << delay << " calculated" << endl;
	      clog << this->fullName() << ", Current completionTime = " << completionTime;
	      clog << ", Current arrivalTime = " << arrivalTime;
	      clog << ", Current delay = " << delay << endl;
	      clog.close();
	      if (arrivalTime > completionTime)
		completionTime = arrivalTime + delay;
	      else
		completionTime += delay;
	      ofstream outFile(filename, ios::app);
	      if (!outFile)
		{
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	  	Error::abortRun(*this,"Unable to open output file.");
		}
	      outFile << "P" << processorNumber << " " << resourceBlockPtr->functionName() << " " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      // send message out acknowledge port
	      dest->put(completionTime) << pkt;
	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", acknowledge to process " << resourceBlockPtr->vectorSize();
	      clog << " particles (" << resourceBlockPtr->cycles() << " cycles, " << resourceBlockPtr->memory();
	      clog << " bytes) sent on acknowledge " << i << " at time " << completionTime << endl;
	      clog.close();
	      }
	    }
}

// prototype instance for known block list
static DEProcessor proto;
static RegisterBlock registerBlock(proto,"Processor");
