static const char file_id[] = "ptclAppInit.cc";
/*******************************************************************
SCCS Version identification :
@(#)ptclAppInit.cc	1.11	9/1/96

Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

						PT_COPYRIGHT_VERSION_2
						COPYRIGHTENDKEY

 Programmer: Wan-Teh Chang
 Date of creation: 10/31/95

This file is adapted from the tclAppInit.c files in the tcl7.4p1 and
tcl7.5a2 releases.  (The original copyright notice in tclAppInit.c is
included below.)  tclAppInit.c is modified into a C++ file, and #ifdef's
are added to make it work with both tcl7.4p1 and tcl7.5a2.  We added
the [incr Tcl] (itcl) extension and the Ptolemy/PTcl facilities.

*******************************************************************/

/* 
 * tclAppInit.c --
 *
 *	Provides a default version of the main program and Tcl_AppInit
 *	procedure for Tcl applications (without Tk).
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "tcl.h"
#ifndef PT_NO_ITCL
#include "itcl.h"
#endif
#include "Linker.h"

#define PIPEPTCL

EXTERN int		Ptcl_Init _ANSI_ARGS_((Tcl_Interp *interp));

// We need this so we can include HOF in ptcl
Tcl_Interp *ptkInterp;

/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */

EXTERN int matherr();
/* This line produces a warning under hppa.cfront:
 *  warning: pointer to function cast to pointer to non-function (149)
 */
int *tclDummyMathPtr = (int *) matherr;

#if TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5
#ifdef TCL_TEST
EXTERN int		TclTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
#endif /* TCL_TEST */
#endif /* TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5 */

#ifdef PIPEPTCL
static Tcl_Interp *interp;      /* Interpreter for application. */
static Tcl_DString command;     /* Used to buffer incomplete commands being */

/*
 *----------------------------------------------------------------------
 *
 * pipeptclTcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

void
pipeptclTcl_Main(int argc,char**  argv, Tcl_AppInitProc* appInitProc)
{
    char buffer[1000], *cmd, *args, *fileName;
    int code, gotPartial, tty;
    int exitCode = 0;

    interp = Tcl_CreateInterp();
#ifdef TCL_MEM_DEBUG
    Tcl_InitMemory(interp);
    Tcl_CreateCommand(interp, "checkmem", CheckmemCmd, (ClientData) 0,
            (Tcl_CmdDeleteProc *) NULL);
#endif

    /*
     * Make command-line arguments available in the Tcl variables "argc"
     * and "argv".  If the first argument doesn't start with a "-" then
     * strip it off and use it as the name of a script file to process.
     */

    fileName = NULL;
    if ((argc > 1) && (argv[1][0] != '-')) {
        fileName = argv[1];
        argc--;
        argv++;
    }
    args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    sprintf(buffer, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buffer, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", (fileName != NULL) ? fileName : argv[0],
            TCL_GLOBAL_ONLY);

    /*
     * Set the "tcl_interactive" variable.
     */

    // force ptcl to run in the interactive mode so that it 
    // pipe communication will work
    tty = 1;
    Tcl_SetVar(interp, "tcl_interactive",
            ((fileName == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);

    /*
     * Invoke application-specific initialization.
     */

    if ((*appInitProc)(interp) != TCL_OK) {
        fprintf(stderr, "application-specific initialization failed: %s\n",
                interp->result);
    }

    /*
     * If a script file was specified then just source that file
     * and quit.
     */

    if (fileName != NULL) {
        code = Tcl_EvalFile(interp, fileName);
        if (code != TCL_OK) {
            fprintf(stderr, "%s\n", interp->result);
            exitCode = 1;
        }
        goto done;
    }

    /*
     * We're running interactively.  Source a user-specific startup
     * file if the application specified one and if the file exists.
     */

#if TCL_MAJOR_VERSION <= 7 && TCL_MINOR_VERSION <= 4
   if (tcl_RcFileName != NULL) {
        Tcl_DString buffer;
        char *fullName;
        FILE *f;

        fullName = Tcl_TildeSubst(interp, tcl_RcFileName, &buffer);
        if (fullName == NULL) {
            fprintf(stderr, "%s\n", interp->result);
        } else {
            f = fopen(fullName, "r");
            if (f != NULL) {
                code = Tcl_EvalFile(interp, fullName);
                if (code != TCL_OK) {
                    fprintf(stderr, "%s\n", interp->result);
                }
                fclose(f);
            }
        }
        Tcl_DStringFree(&buffer);
    }
#endif
    /*
     * Process commands from stdin until there's an end-of-file.
     */

    gotPartial = 0;
    Tcl_DStringInit(&command);
    while (1) {
        clearerr(stdin);
        if (tty) {
            char *promptCmd;

            promptCmd = Tcl_GetVar(interp,
                gotPartial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
            if (promptCmd == NULL) {
                defaultPrompt:
                if (!gotPartial) {
                    fputs("% ", stdout);
                }
            } else {
                code = Tcl_Eval(interp, promptCmd);
                if (code != TCL_OK) {
                    fprintf(stderr, "%s\n", interp->result);
                    Tcl_AddErrorInfo(interp,
                            "\n    (script that generates prompt)");
                    goto defaultPrompt;
                }
            }
            fflush(stdout);
        }
        if (fgets(buffer, 1000, stdin) == NULL) {
            if (ferror(stdin)) {
                if (errno == EINTR) {
#if TCL_MAJOR_VERSION <= 7 && TCL_MINOR_VERSION <= 4
                    if (tcl_AsyncReady) {
                        (void) Tcl_AsyncInvoke((Tcl_Interp *) NULL, 0);
                    }
#else
 		    if ( Tcl_AsyncReady() ) {
 			(void) Tcl_AsyncInvoke((Tcl_Interp *) NULL, 0);
 		    }
#endif

                    clearerr(stdin);
                } else {
                    goto done;
                }
            } else {
                if (!gotPartial) {
                    goto done;
                }
            }
            buffer[0] = 0;
        }
        cmd = Tcl_DStringAppend(&command, buffer, -1);
        if ((buffer[0] != 0) && !Tcl_CommandComplete(cmd)) {
            gotPartial = 1;
            continue;
        }


        gotPartial = 0;
        code = Tcl_RecordAndEval(interp, cmd, 0);
        Tcl_DStringFree(&command);
        if (code != TCL_OK) {
            fprintf(stderr, "%s\n", interp->result);
        } else if (tty && (*interp->result != 0)) {
            /*
             * Important:  use puts and not printf here.  On Solaris 2.3
             * and 2.4, and perhaps other systems, printf will dump core
             * if interp->result is big.
             */

            puts(interp->result);
        }
#ifdef TCL_MEM_DEBUG
        if (quitFlag) {
            Tcl_DeleteInterp(interp);
            Tcl_DumpActiveMemory(dumpFile);
            exit(0);
        }
#endif
    }

    /*
     * Rather than calling exit, invoke the "exit" command so that
     * users can replace "exit" with some other command to do additional
     * cleanup on exit.  The Tcl_Eval call should never return.
     */

    done:
    sprintf(buffer, "exit %d", exitCode);
    Tcl_Eval(interp, buffer);
}
#endif //PIPEPTCL


/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Arguments:
 *	argc: Number of command-line arguments.
 *	argv: Values of command-line arguments.
 *
 * Results:
 *	None: Tcl_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

int main(int argc, char **argv)
{
    // Initialize the Ptolemy incremental linker module.
    // Note: this initialization routine must be invoked here in the
    // main() function rather than in Ptcl_Init() because it needs to
    // be given the name of the binary executable (argv[0]).
    // (Ptcl_Init() does not have access to argv.)
    Linker::init(argv[0]);

#ifdef PIPEPTCL
    pipeptclTcl_Main(argc, argv, Tcl_AppInit);
#else
    Tcl_Main(argc, argv, Tcl_AppInit);
#endif
    return 0;			/* Needed only to prevent compiler warning. */
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Arguments:
 *	interp: Interpreter for application.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AppInit(Tcl_Interp *interp)
{
    // We need this so that we can use hof inside ptcl
    ptkInterp=interp;
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

#if TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5
#ifdef TCL_TEST
    if (TclTest_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
#endif /* TCL_TEST */
#endif /* TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5 */

    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */

#ifndef PT_NO_ITCL
    // Add [incr Tcl] (itcl) facilities
    if (Itcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
#endif
    // Add PTcl commands
    if (Ptcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

#ifdef PIPEPTCL
    // Set tcl variable tcl_interactive to 1 to force interactivity
    Tcl_SetVar(interp, "tcl_interactive", "1", TCL_GLOBAL_ONLY);
#if TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5
     // Add 'flush stdout' to the commands that print the prompts so that
     // weX actually get the prompt.  Otherwise, pigi+ just hangs.
     Tcl_SetVar(interp, "tcl_prompt1",
              "puts -nonewline stdout {ptcl> }; flush stdout",
              TCL_GLOBAL_ONLY);
     Tcl_SetVar(interp, "tcl_prompt2",
              "puts -nonewline stdout {ptcl? }; flush stdout",
              TCL_GLOBAL_ONLY);
#endif
#endif

    /*
     * Call Tcl_CreateCommand for application-specific commands, if
     * they weren't already created by the init procedures called above.
     */

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

#if TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5
    Tcl_SetVar(interp, "tcl_rcFileName", "~/.ptclrc", TCL_GLOBAL_ONLY);
#else
    tcl_RcFileName = "~/.ptclrc";
#endif /* TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION >= 5 */
    return TCL_OK;
}
