defstar
{
	name { ReadComplexFile }
	domain { SDF }
	version { $Revision: 1.4 $ $Date: 1996/04/22 18:54:16 $ }
	author { E. K. Pauer, adapted from SDFReadFile.pl by T. M. Parks }
	copyright {
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
	}
        acknowledge {
This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
}
	location { $PTOLEMY/src/contrib/atrade/sdf/stars }
	desc { Reads complex data from a file.  The simulation can be halted
	on end-of-file, or the file contents can be periodically repeated,
	or the file contents can be padded with zeros.
	}
	location { ~ptolemy/src/domains/sdf/stars }
	
    	hinclude { <fstream.h> }
    	ccinclude { "SimControl.h" }

    	output {
		name { output }
		type { complex }
    	}
	defstate {
		name { fileName }
		type { string }
		default { "/dev/null" }
		descriptor { Input file }
    	}
    	defstate {
		name { haltAtEnd }
		type { int }
		default { "NO" }
		descriptor { Halt the run at the end of the input file. }
    	}
    	defstate {
		name { periodic }
		type { int }
		default { "YES" }
		descriptor { Make output periodic or zero-padded. }
    	}
        defstate {
		name { blockSize }
		type { int }
		default { 256 }
		descriptor { Blocksize }
	}
        defstate {
		name { frequency }
		type { float }
		default { 0.0 }
		descriptor { Frequency at which blocks will be generated }
	}
	protected {
	istream* input;
	}

	setup {
	// open input file
	LOG_NEW; input = new ifstream(expandPathName(fileName));
	if (!(*input))
		Error::abortRun(*this, "can't open file ", fileName);
	output.setSDFParams(int(blockSize), int(blockSize)-1);
    	}

	go {
	  double x[2];
	  x[0] = 0.0;
	  x[1] = 0.0;

          for (int s = int(blockSize)-1; s >= 0; s--)
	    {
	    for (int i = 0; i < 2; i++)
              {
	      if (input->eof())
	        {
	        if (haltAtEnd)		// halt the run
		  SimControl::requestHalt();
	        else if (periodic)		// close and re-open file
		  {
		  LOG_DEL; delete input;
		  LOG_NEW; input = new ifstream(expandPathName(fileName));
		  if (!(*input))
                    {
		    Error::abortRun(*this, "can't re-open file ", fileName);
		    return;
		    }
		  (*input) >> x[i];		// get next value
		  ws(*input);
		  }
	        }
	      else			// get next value
	        {
	        (*input) >> x[i];
	        ws(*input);
	        }
              }
	    output%s << Complex(x[0], x[1]);
	    }
	}

	wrapup {
	LOG_DEL; delete input;
	}
}
